function [wtS,wtC,wtP,td,F_last] = plsa_onlyS(shape_id,color_id,pos_id,T,epsilon,sizeS,sizeC,sizeP,plo,learn)
%
% learns plsa model with T topics from words x docs counts data
% 
% counts  (W x D)  : matrix with count data of W words for D documents 
% T       (scalar) : desired number of topics 
% epsilon (scalar) : EM iterations terminate if the relative change in log-likelihood is under epsilon
% plo     (scalar) : plot log-likelihood progress and wt and td matrices if plo ~= 0
% wt      (W x T)  : optional, initializes word-topic distributions from wt (columns should normalize to one) 
% learn   (scalar) : keeps wt fixed if learn = 0;
%
% wt      (W x T)  : word-topic distributions
% td      (T x D)  : topic-document distributions
% E	  (1 x D)  : document log-likelihood: log p( words in doc | doc )
%
% J.J. Verbeek,o INRIA Rhone-Alpes, 2006. (adapted J. van de Weijer 2007)

if(plo), COLORMAP('default'); end;

D=size(shape_id,2);

fprintf('--> PLSA started with %d topics, %d documents, and (%d,%d,%d) word vocabulary.\n',T,1,sizeS,sizeC,sizeP);

if nargin<10; learn = 0; end

if learn==1
    wtS= normalize( rand(sizeS,T), 1);
    wtC= normalize( rand(sizeC,T), 1);
    wtP= normalize( rand(sizeP,T), 1);
end

%td   = normalize( ones(T,D), 1);

done = 0;
iter = 1;
td=ones(T,D);

td=normalize(td,1);

for ii=1:D    
    ctP{ii}=wtP(pos_id{ii},:);
    ctS{ii}=wtS(shape_id{ii},:);
    ctC{ii}=wtC(color_id{ii},:);
    ct{ii}=ctS{ii};                      % we only take shape into account
    tot{ii}=length(pos_id{ii});
    
    cw_p{ii}=sparse((1:tot{ii}),pos_id{ii},ones(1,tot{ii}),tot{ii},sizeP);
    cw_s{ii}=sparse((1:tot{ii}),shape_id{ii},ones(1,tot{ii}),tot{ii},sizeS);
    cw_c{ii}=sparse((1:tot{ii}),color_id{ii},ones(1,tot{ii}),tot{ii},sizeC);
end



E=0;
for ii=1:D
    E=E+sum(log(ct{ii}*td(:,ii)+eps));
end
   
F = sum( E ) / D;

while ~done; iter = iter+ 1; 
   
   
   for ii=1:D
       td(:,ii)= td(:,ii) .* ( ct{ii}' * ( ones(tot{ii},1) ./ ( ct{ii} * td(:,ii) + eps ) ) );
   end
   
   td=normalize(td,1);
      if learn;
       
       wtS2=zeros(size(wtS));
       wtC2=zeros(size(wtC));
       wtP2=zeros(size(wtP));
       for ii=1:D
           ctT= normalize(ct{ii}.*(ones(size(ct{ii},1),1)*td(:,ii)'),2)/tot{ii};
           wtS2=wtS2+cw_s{ii}'*ctT;
           wtC2=wtC2+cw_c{ii}'*ctT;
           wtP2=wtP2+cw_p{ii}'*ctT;
       end
       wtS=normalize(wtS2,1);
       wtC=normalize(wtC2,1);
       wtP=normalize(wtP2,1);
       
       for ii=1:D
            ctS{ii}=wtS(shape_id{ii},:);
            ctC{ii}=wtC(color_id{ii},:);
            ctP{ii}=wtP(pos_id{ii},:);
            ct{ii}=ctS{ii};                         % we only take shape into account
       end
   end

   E=0;
   for ii=1:D
       E=E+sum(log(ct{ii}*td(:,ii)+eps));
   end

   F_new = sum( E ) / D; F =  [F F_new];
       
   rel_ch = (F(iter) - F(iter-1))/abs(F(iter-1)); if rel_ch< epsilon && (iter> 5 ); done=1;end
   fprintf('iteration %3d F = %f rel-ch = %.6f \n', iter, F(iter), rel_ch); 
  
   if plo;
      Nplots = 5;    
      set(gcf,'Name','Probabilistic Latent Semantic Analysis');
      subplot(Nplots,1,1); imagesc(td);         title('Distribution over topics per document');
      subplot(Nplots,1,2); imagesc(wtS');       title('SHAPE:Distribution over words per topic');
      subplot(Nplots,1,3); imagesc(wtC');       title('COLOR:Distribution over words per topic');
      subplot(Nplots,1,4); imagesc(wtP');       title('POS:Distribution over words per topic');
      subplot(Nplots,1,5); plot(F(2:end),'.-'); title('Average data log-likelihood per iteration')
      drawnow;
   end   
end
F_last=E;
